/********************************************************************
 FileName:     	HardwareProfile - PIC18F46J50 PIM.h
 Dependencies:	See INCLUDES section
 Processor:	PIC18 USB Microcontrollers
 Hardware:	PIC18F46J50 PIM
 Compiler:  	Microchip C18
 Company:	Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the CompanyE for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
  2.3   09/15/2008   Broke out each hardware platform into its own
                     "HardwareProfile - xxx.h" file
********************************************************************/

#ifndef HARDWARE_PROFILE_PIC18F46J50_PIM_H
#define HARDWARE_PROFILE_PIC18F46J50_PIM_H

    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //#define USE_SELF_POWER_SENSE_IO
    #define tris_self_power     TRISCbits.TRISC2    // Input
    #define self_power          1

    //#define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  TRISCbits.TRISC2    // Input
    #define USB_BUS_SENSE       1 

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

    #define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER

    /** Board definition ***********************************************/
    //These defintions will tell the main() function which board is
    //  currently selected.  This will allow the application to add
    //  the correct configuration bits as wells use the correct
    //  initialization functions for the board.  These defitions are only
    //  required in the stack provided demos.  They are not required in
    //  final application design.
    #define DEMO_BOARD PIC18F46J50_PIM
    #define PIC18F46J50_PIM
    #define CLOCK_FREQ 48000000
    #define GetSystemClock() CLOCK_FREQ   

    /** LED ************************************************************/


/*
// Port Assignment
 

 RA0		O	
 RA1		O	LED1
 RA2		O	LED2
 RA3		O	LED3
 RA4		O	NC
 RA5		O	LCD_CS
 RA6			OSC2
 RA7			OSC1
 RB0		I	VS (INT)
 RB1		O	CAMWEN
 RB2		O	CAMRCLK
 RB3		O	CAMOE
 RB4		O	SCL
 RB5		O	SDA
 RB6		I	PGC
 RB7		I 	PGD (TEST_SW) 
 RC0		O	LCD_RST
 RC1		O		
 RC2		O	
 RC3			-
 RC4			D-
 RC5			D+
 RC6		O	LCD_CD
 RC7		O	CAMFRST

 RD0-RD7	B	D0-D7
 RE0		O	RD
 RE1		O	WR
 RE2		O	

*/


#define Fosc 48
// LCD Ports define 
#define LCD_TRIS 	TRISD
#define	LCD_DIO		LATCbits.LATC6
#define	LCD_CS		LATAbits.LATA5
#define LCD_WR		LATEbits.LATE1
#define LCD_RD		LATEbits.LATE0
#define LCD_RST		LATCbits.LATC0
#define LED1		LATAbits.LATA1
#define LED2		LATAbits.LATA2
#define LED3		LATAbits.LATA3

#define MALWR		LATCbits.LATC2
#define MAHWR		LATCbits.LATC1

#define RAMCE		LATEbits.LATE2
#define MA16		LATAbits.LATA0

#define VS			LATBbits.PORTB0
#define CAMFRST		LATCbits.LATC7
#define CAMWEN		LATBbits.LATB1
#define CAMRCLK		LATBbits.LATB2
#define CAMOE		LATBbits.LATB3

#define SCCB_SIC	LATBbits.LATB4
#define	SCCB_SID	LATBbits.LATB5
#define	SCCB_SID_DIR	TRISBbits.TRISB5
#define SCCB_SID_IN 	PORTBbits.RB5



    #define mInitAllLEDs()      LATA &= 0xF3; TRISE &= 0xF3;
    
    #define mLED_1              LED1
    #define mLED_2              LED2
    #define mLED_3              LED3
    #define mLED_4              
    
    #define mGetLED_1()         mLED_1
    #define mGetLED_2()         mLED_2
    #define mGetLED_3()         1
    #define mGetLED_4()         1

    #define mLED_1_On()         mLED_1 = 1;
    #define mLED_2_On()         mLED_2 = 1;
    #define mLED_3_On()         
    #define mLED_4_On()        
    
    #define mLED_1_Off()        mLED_1 = 0;
    #define mLED_2_Off()        mLED_2 = 0;
    #define mLED_3_Off()        
    #define mLED_4_Off()        
    
    #define mLED_1_Toggle()     mLED_1 = !mLED_1;
    #define mLED_2_Toggle()     mLED_2 = !mLED_2;
    #define mLED_3_Toggle()     
    #define mLED_4_Toggle()     
   
    /** SWITCH *********************************************************/
/*
    #define mInitSwitch2()      TRISBbits.TRISB2=1;
    #define mInitSwitch3()      mInitSwitch2();
    #define mInitAllSwitches()  mInitSwitch2();
    #define sw2                 PORTBbits.RB2
    #define sw3                 PORTBbits.RB2
*/
	/** RS 232 lines ****************************************************/
/*
	#define UART_TRISTx   TRISCbits.TRISC6
	#define UART_TRISRx   TRISCbits.TRISC7
	#define UART_Tx       PORTCbits.RC6
	#define UART_Rx       PORTCbits.RC7
	#define UART_TRISRTS  TRISBbits.TRISB1
	#define UART_RTS      PORTBbits.RB1
	#define UART_TRISDTR  TRISBbits.TRISB2
	#define UART_DTR      PORTBbits.RB2
    #define UART_ENABLE RCSTAbits.SPEN
*/
    /** I/O pin definitions ********************************************/
/* 
   #define INPUT_PIN 1
    #define OUTPUT_PIN 0
*/
#endif  //HARDWARE_PROFILE_PIC18F46J50_PIM_H
