/*
 * Measure distance to Wii IR sensor bar
 */
#include <stdlib.h>
#include <string.h>
#include "SAMCamera.h"
#include <stdio.h>
#include "print.h"

typedef union {
  struct s_pl {
    uint8_t pixel:1;
    uint8_t label:7;
  } __attribute__((packed)) pix_label;
  uint8_t yval;
} __attribute__((packed)) IMAGEDATA;

typedef struct {
  short xpos, ypos;
  short width, height;
  short area;
  short minx, maxx;
  short miny, maxy;
} LABELPROP;

LABELPROP sensors[2];

int get_label_property(OV_INFO *pinfo, IMAGEDATA *pImg, int label, LABELPROP *pProp)
{
  int minx, miny, maxx, maxy;
  int area;
  int x, y;

  minx = pinfo->width;
  miny = pinfo->height;
  maxx = maxy = 0;
  pProp->xpos = pProp->ypos = 0;
  pProp->width = pProp->height = 0;
  area = 0;
  for (y = 0; y < pinfo->height;y++) {
    for (x = 0; x < pinfo->width; x++) {
      if (pImg->pix_label.label == label) {
        if (x < minx) minx = x;
        if (x > maxx) maxx = x;
        if (y < miny) miny = y;
        if (y > maxy) maxy = y;
        area++;
      }
      pImg++;
    }
  }
  if (area > 0) {
    pProp->area = area;
    pProp->width = maxx - minx + 1;
    pProp->height = maxy - miny + 1;
    pProp->xpos = minx + pProp->width/2;
    pProp->ypos = miny + pProp->height/2;
#ifdef XDEBUG
    cons_printf("%d: (%d, %d) --> %d X %d\r\n", label, pProp->xpos, pProp->ypos, pProp->width, pProp->height);
#endif
  }
  return area;
}

int labeling(IMAGEDATA *pImg, OV_INFO *pinfo, int *pDeleted)
{
  int x, y, x2, y2;
  int lambda, lambda1, lambda2;
  int l1, l2, l3, l4;
  int sw, deleted;
  IMAGEDATA *wp;

  deleted = lambda = 0;

  for (y = 1; y < pinfo->sc_height; y++) {
    for (x = 1; x < pinfo->width - 1; x++) {
      wp = pImg + pinfo->width * y + x;
      if (wp->pix_label.pixel) {	// Black
        l1 = wp[-pinfo->width-1].pix_label.label;
        l2 = wp[-pinfo->width].pix_label.label;
        l3 = wp[-pinfo->width+1].pix_label.label;
        l4 = wp[-1].pix_label.label;

        if (l1 == 0 && l2 == 0 && l3 == 0 && l4 == 0) {
          if (lambda < 127) lambda++;
          wp[0].pix_label.label = lambda;
        } else {
          lambda1 = l1;
          lambda2 = -1;
          if (l1 != l2) lambda2 = l2;
          else if (l1 != l3) lambda2 = l3;
          else if (l1 != l4) lambda2 = l4;
          if (lambda1 > lambda2) {
            sw = lambda1;
            lambda1 = lambda2;
            lambda2 = sw;
          }
          if (lambda1 == -1 || lambda1 == 0) {
            wp[0].pix_label.label = lambda2;
          } else {
            wp[0].pix_label.label = lambda1;
            for (y2 = 0; y2 <= y; y2++) {
              for (x2 = 0; x2 < pinfo->width; x2++) {
                if (pImg[pinfo->width*y2+x2].pix_label.label == lambda2)
                  pImg[y2*pinfo->width+x2].pix_label.label = lambda1;
              }
            }
            deleted++;
          }
        }
      } else {
        wp[0].pix_label.label = 0;
      }
    }
  }
#ifdef XDEBUG
  cons_printf("lambda = %d, deleted = %d\r\n", lambda, deleted);
#endif
  *pDeleted = deleted;
  return lambda;
}

/*
 * Find biggest two regions
 */
void find_region(IMAGEDATA *pImg, OV_INFO *pinfo, int max_label)
{
  LABELPROP prop, *p0, *p1;
  int area, i;

  p0 = &sensors[0];
  p1 = &sensors[1];
  p0->area = p1->area = 0;

  for (i = 1; i <= max_label; i++) {
    area = get_label_property(pinfo, pImg, i, &prop);
    if (area > p0->area) {
      if (area > p1->area) {
        *p0 = *p1;
        *p1 = prop;
      } else {
        *p0 = prop;
      }
    }
  }
}

void get_region(IMAGEDATA *pImg, OV_INFO *pinfo)
{
  LABELPROP *pProp;
  int x, y;
  int label, l1;
  IMAGEDATA *wp;

  sensors[0].minx = sensors[1].minx = pinfo->width ;
  sensors[0].maxx = sensors[1].maxx = 0;
  sensors[0].miny = sensors[1].miny = pinfo->sc_height;
  sensors[0].maxy = sensors[1].maxy = 0;

  /* Find one of two label numbers */
  wp = pImg;
  for (x = 0; x < pinfo->sc_height * pinfo->width; x++) {
    label = wp->pix_label.label;
    if (label > 0) {
      l1 = label;
      break;
    }
    wp++;
  }

  for (y = 1; y < pinfo->sc_height; y++) {
    wp = pImg + pinfo->width * y + 1;
    for (x = 1; x < pinfo->width - 1; x++) {
      label = wp->pix_label.label;
      if (label > 0) {
        if (label == l1)
          pProp = &sensors[0];
        else
          pProp = &sensors[1];
        if (x < pProp->minx) pProp->minx = x;
        if (y < pProp->miny) pProp->miny = y;
        if (x > pProp->maxx) pProp->maxx = x;
        if (y > pProp->maxy) pProp->maxy = y;
      }
      wp++;
    }
  }
  for (pProp = sensors; pProp < &sensors[2]; pProp++) {
    pProp->xpos = pProp->minx + (pProp->maxx - pProp->minx)/2;
    pProp->ypos = pProp->miny + (pProp->maxy - pProp->miny)/2;
  }
}

int find_sensor(IMAGEDATA *base, OV_INFO *pinfo)
{
  int i, ygood, label_count;
  int deleted;
  IMAGEDATA *img = base;
  uint8_t yval, ymax;

  label_count = ygood = 0;
  ymax = 0;

  for (i = 0; i < pinfo->width * pinfo->sc_height; i++) {
    yval = img->yval;
    if (yval > ymax) ymax = yval;
    if (yval >= 240) {
      img->pix_label.pixel = 1;
      ygood++;
    } else {
      img->pix_label.pixel = 0;
    }
    img->pix_label.label = 0;
    img++;
  }
#ifdef XDEBUG
  cons_printf("ymax = %d, ygood = %d\r\n", ymax, ygood);
#endif
  if (ygood > 1) {
    label_count = labeling(base, pinfo, &deleted);
    if (label_count - deleted >= 2) {
      find_region(base, pinfo, label_count);
    }
    label_count -= deleted;
  }
  return label_count;
}

int const_a = 590;

/*
    dx:f = Dx : l
    l = (f * Dx)/dx
    const_a = f * Dx = l * dx
 */
void measure_distance(OV_INFO *pinfo)
{
  int n, dx, dy, midx, mdiff;
  int val, color;
  char str_buffer[40];

  n = find_sensor((IMAGEDATA *)ov_buffer, pinfo);
  if (n >= 2) {
     dx = sensors[1].xpos - sensors[0].xpos;
     if (dx < 0) dx = -dx;
     if (pinfo->navsw & PIO_PB0) { /* Calibrate at 1000mm distance */
       const_a = dx * 1000;
     }
     midx = (sensors[0].xpos + sensors[1].xpos)/2;
     mdiff = midx - pinfo->width/2;
     if (mdiff < 0) mdiff = -mdiff;
     dy = sensors[1].ypos - sensors[0].ypos;
     if (dy < 0) dy = -dy;
     cons_printf("%d(%d), %d\r\n", midx, mdiff, dy);
     if (dy < 2) {
       if (mdiff < 5)
         color = C_GREEN;
       else if (mdiff < 8)
         color = C_YELLOW;
       else
         color = C_RED;
     } else if (dy < 3)
       color = C_YELLOW;
     else
       color = C_RED;
     val = const_a/dx;
     sprintf(str_buffer, "dx: %3d dist: %5d", dx, val);
     lcd_string(str_buffer, FT_SMALL, 0, 1, color, C_BLACK);
  }
}
