/*
 * Number reader demo application using OV7670
 *
 * Written by Toshiharu Ohno (sirius506@gmail.com)
 */
#include <string.h>
#include <stdlib.h>
#include "SAMCamera.h"
#include "usb.h"

extern void setup_mode(ov_configs *pconfigs, OV_INFO *pinfo);
extern void disp_camera_mode(OV_INFO *pinfo);

void application_task()
{
  OV_INFO *pinfo = &ov_info;

  nav_pio_init();
  ov_set_resolution(RES_QCIF);

  pinfo->width = OV_COLS;
  pinfo->height = OV_LINES;
  pinfo->sc_width = OV_COLS;
  pinfo->sc_height = OV_LINES;
  pinfo->status = ST_PREVIEW;
  pinfo->event = 0;
  pinfo->navsw = 0;

  udp_disable();
  delay_ms(20);

  pinfo->navsw = 0;
  pinfo->event = 0;

  lcd_fill_rect(0, 0, 160, 128, C_BLACK);
  disp_camera_mode(pinfo);

  ov_start(pinfo);

  udp_init();

  for (;;) {
    /*
     * Wait for some events
     */
    while (pinfo->event == 0) {
      usb_task();
    }
    if (pinfo->event & CAMEV_DMA_DONE) {
      /* new image frame becomes ready,
       * transfer it to the LCD.
       */
      lcd_copy_frame(ov_buffer, pinfo->width, pinfo->height);
      pinfo->dma_busy = 0;
      pinfo->copy_count++;
    }
    if (pinfo->navsw) {
      ov_stop(pinfo);
      if (pinfo->navsw & (PIO_PB0|PIO_PB2)) {
        number_decode(pinfo);
      } else if (pinfo->navsw & PIO_PB1) {
        setup_mode(pconfigs, pinfo);
      }
      ov_restart(pinfo);
      pinfo->navsw = 0;
    }
    pinfo->event = 0;
  }
}
