/*
 * Setup screen for Number Reader application
 */
#include <string.h>
#include <stdlib.h>
#include "SAMCamera.h"

#define	NUM_EFFECT  11

typedef struct {
  char *resolution_name;
  RESOLUTION  resolution_val;
} resolution_def;

const resolution_def OVResolutions[NUM_RESOLUTION] = {
  { "QCIF", RES_QCIF },
  { "QVGA", RES_QVGA },
  { "VGA ", RES_VGA },
};

typedef struct {
  char *effect_name;
  IEFFECT  effect_val;
} effect_def;

const effect_def DigitalEffects[NUM_EFFECT] = {
 { "Normal  ", IE_NORMAL },
 { "Sepia   ", IE_SEPIA },
 { "Blush   ", IE_BLUSH },
 { "Reddish ", IE_REDDISH },
 { "Greenish", IE_GREENISH },
 { "Negative", IE_NEGATIVE },
 { "X-Ray   ", IE_XRAY },
 { "Night0  ", IE_NIGHT0 },
 { "Night1  ", IE_NIGHT1 },
 { "Night2  ", IE_NIGHT2 },
 { "Night3  ", IE_NIGHT3 },
};

typedef struct {
  short xpos, vxpos, ypos;
  char *title;
  const char *(*func_mstr)(OV_INFO *pinfo);
} menu_title;

static const char *get_resolution_string(OV_INFO *pinfo)
{
  return OVResolutions[pinfo->resolution].resolution_name;
}

static const char *get_mode_string(OV_INFO *pinfo)
{
  return (pinfo->direct_mode ? "Direct" : "Step  ");
}

static const char *get_effect_string(OV_INFO *pinfo)
{
  return DigitalEffects[pinfo->effect].effect_name;
}

menu_title Menu_Titles[5] = {
 { 50, 85,  15, "Setup",  NULL                    },	/* 0 */
 { 20, 80,  40, "Res",    get_resolution_string   },	/* 1 */
 { 20, 80,  60, "Mode",   get_mode_string         },	/* 2 */
 { 20, 80,  80, "Effect", get_effect_string       },	/* 3 */
 { 20, 80, 100, "Quit",   NULL                    },	/* 4 */
};

void disp_camera_mode(OV_INFO *pinfo)
{
  char *str;

  str = (char *)get_resolution_string(pinfo);
  lcd_string(str, FT_SMALL, 0, 1, C_YELLOW, C_BLACK);
  str = (char *)get_mode_string(pinfo);
  lcd_string(str, FT_SMALL, 55, 1, C_YELLOW, C_BLACK);
  str = (char *)get_effect_string(pinfo);
  lcd_string(str, FT_SMALL, 106, 1, C_YELLOW, C_BLACK);
}

void setup_mode(ov_configs *pconfigs, OV_INFO *pinfo)
{
  int cpos;
  IEFFECT ceffect, neweffect;
  RESOLUTION cres;
  int i;
  const char *str;
  menu_title *mp;

  /* Draw menu */
  lcd_fill_rect(0, 0, 160, 130, C_BLACK);
  mp = Menu_Titles;
  for (i = 0; i < 5; i++) {
    lcd_string(mp->title,  FT_LARGE, mp->xpos, mp->ypos, C_WHITE, C_BLACK);
    if (mp->func_mstr) {
      str = (mp->func_mstr)(pinfo);
      lcd_string((char *)str, FT_LARGE, mp->vxpos, mp->ypos, C_WHITE, C_BLACK);
    }
    mp++;
  }

  cpos = 1;	/* current menu cursor position */
  cres = pinfo->resolution;
  ceffect = pinfo->effect;

  while (1) {
    /* Draw inverted menu string where cursor is located */
    mp = Menu_Titles + cpos;
    lcd_string((char *)mp->title,  FT_LARGE, mp->xpos, mp->ypos, C_BLACK, C_WHITE);

    /* Wait for button action */
    pinfo->navsw = 0;
    while (pinfo->navsw == 0)
      delay_ms(10);

    if (pinfo->navsw & PIO_PB1) {	/* Push */
      switch (cpos) {
      case 1:	/* Resolution */
        pinfo->resolution++;
        if (pinfo->resolution == NUM_RESOLUTION)
          pinfo->resolution = RES_QCIF;
        break;
      case 2:	/* Mode */
        pinfo->direct_mode ^= 1;
        break;
      case 3:	/* Effect */
        pinfo->effect++;
        if (pinfo->effect == NUM_EFFECT)
          pinfo->effect = IE_NORMAL;
        break;
      case 4:	/* Quit */
        if (pinfo->resolution != cres) {
          lcd_fill_rect(0, 0, 160, 120, C_BLACK);
          lcd_string("Wait a moment..", FT_LARGE, 20, 50, C_GREEN, C_BLACK);
          ov_set_resolution(pinfo->resolution);
          ceffect = IE_NORMAL;
        } else {
          pinfo->resolution = cres;
        }
        if (pinfo->effect != ceffect) {
          neweffect = DigitalEffects[pinfo->effect].effect_val;
          (pconfigs->image_effect)(neweffect);
        }
        disp_camera_mode(pinfo);
        return;
      }
      /* display newly selected value */
      str = (mp->func_mstr)(pinfo);
      lcd_string((char *)str, FT_LARGE, mp->vxpos, mp->ypos, C_WHITE, C_BLACK);
    } else {
      /* Up/down action. Redraw current menu with non-inverted string */
      lcd_string(mp->title,  FT_LARGE, mp->xpos, mp->ypos, C_WHITE, C_BLACK);
      if (pinfo->navsw & PIO_PB0) {	/* Up */
        cpos--;
        if (cpos == 0) cpos = 4;
      } else if (pinfo->navsw & PIO_PB2) {	/* Down */
        cpos++;
        if (cpos > 4) cpos = 1;
      }
    }
  }
}
