/*
 * SAM3S TWI driver
 */
#include <string.h>
#include "SAMCamera.h"
#include "i2c.h"

static int txnak_cnt;

static int sem_twi;

void TWI0_IRQHandler()
{
  volatile Twi *pTWI;
  int twi_sr;

  pTWI = TWI0;
  twi_sr = pTWI->TWI_SR & pTWI->TWI_IMR;

  if (twi_sr & TWI_SR_ENDTX) {
    pTWI->TWI_IDR = TWI_IDR_ENDTX;
    pTWI->TWI_PTCR = TWI_PTCR_TXTDIS;
    pTWI->TWI_CR = TWI_CR_STOP;
    pTWI->TWI_IER = TWI_IER_TXCOMP;
  }
  if (twi_sr & TWI_SR_RXRDY) {
    pTWI->TWI_IDR = TWI_IDR_RXRDY;
    pTWI->TWI_IER = TWI_IER_TXCOMP;
    pTWI->TWI_PTCR = TWI_PTCR_RXTDIS;
  }
  if (twi_sr & TWI_SR_TXCOMP) {
    /* TXCOMP is set when STOP has sent or NAK is detected.
     * PDC should be stopped in both cases.
     */
    if (twi_sr & TWI_SR_NACK)
      txnak_cnt++;
    pTWI->TWI_RCR = 0;
    pTWI->TWI_PTCR = TWI_PTCR_RXTDIS | TWI_PTCR_TXTDIS;
    pTWI->TWI_IDR = TWI_IDR_TXCOMP;
    sem_twi = 1;
  }
}

//#define	USE_INTER

uint8_t i2c_read_reg(uint16_t devaddr, uint32_t regno)
{
  uint8_t bval;
  volatile Twi *pTWI = TWI0;
  while (!(pTWI->TWI_SR & TWI_SR_TXCOMP))
    ;
  pTWI->TWI_CR = TWI_CR_SVDIS | TWI_CR_MSEN;
  pTWI->TWI_MMR = 0;
#ifdef USE_INTER
  pTWI->TWI_MMR = TWI_MMR_DADR(devaddr) | TWI_MMR_MREAD | TWI_MMR_IADRSZ_1_BYTE;
  pTWI->TWI_IADR = regno;
  pTWI->TWI_IER = TWI_IER_RXRDY;
  pTWI->TWI_CR = TWI_CR_START | TWI_CR_STOP;
  ctl_semaphore_wait(&sem_twi, CTL_TIMEOUT_NONE, 0);
#else
  pTWI->TWI_MMR = (devaddr << 16);
  pTWI->TWI_THR = regno;
  pTWI->TWI_CR = TWI_CR_STOP;
  while (!(pTWI->TWI_SR & TWI_SR_TXRDY)) ;
  while (!(pTWI->TWI_SR & TWI_SR_TXCOMP));

  pTWI->TWI_MMR = (devaddr << 16) | TWI_MMR_MREAD;
  pTWI->TWI_CR = TWI_CR_START | TWI_CR_STOP;
  while (!(pTWI->TWI_SR & TWI_SR_RXRDY)) ;
  bval = pTWI->TWI_RHR;
  while (!(pTWI->TWI_SR & TWI_SR_TXCOMP));
#endif

  return (bval);
}

void i2c_write_reg(uint16_t devaddr, uint32_t regno, uint8_t bdata)
{
  volatile Twi *pTWI = TWI0;
  uint8_t txbuff[2];

  while (!(pTWI->TWI_SR & TWI_SR_TXCOMP))
    ;
  pTWI->TWI_MMR = (devaddr << 16) | TWI_MMR_IADRSZ_1_BYTE;
  pTWI->TWI_IADR = regno;
  txbuff[0] = bdata;
  pTWI->TWI_TPR = (RwReg) txbuff;
  pTWI->TWI_TCR = 1;
  pTWI->TWI_IER = TWI_IER_ENDTX;
  pTWI->TWI_PTCR = TWI_PTCR_TXTEN;
  while (sem_twi == 0)
    ;
  sem_twi = 0;
}

/*
 * Initialize TWI interace and generate XLCK clock for Omnivision.
 */
void i2c_init(void)
{
  Pio *pPIOA = PIOA;
  Twi *pTWI = TWI0;

  /* Configure pins. Note that we no longer need to
   *  set these pins as open-drain with SAM3!!
   */
  pPIOA->PIO_PDR = PIO_PA3 | PIO_PA4;
  pPIOA->PIO_ODR = PIO_PA3 | PIO_PA4;
  pPIOA->PIO_ABCDSR[0] &= ~(PIO_PA3A_TWD0 | PIO_PA4A_TWCK0);
  pPIOA->PIO_ABCDSR[1] &= ~(PIO_PA3A_TWD0 | PIO_PA4A_TWCK0);

  /*
   * PA6 is used as PCK0 which generates 12MHz.
   * This clock act as OV7670 XCLK.
   */
  pPIOA->PIO_PDR = PIO_PA6;
  pPIOA->PIO_ABCDSR[0] |= PIO_PA6B_PCK0;

  pTWI->TWI_CR = TWI_CR_SWRST;
  /* Set TWI clock (CLDIV, CHDIV, CKDIV) in TWI_CWGR */
  pTWI->TWI_CWGR = (3 << 16) | (80 << 8) | (80 << 0);

  /* Enable master mode and disable slave */
  pTWI->TWI_CR = TWI_CR_SVDIS;
  pTWI->TWI_CR = TWI_CR_MSEN;

  NVIC_EnableIRQ(TWI0_IRQn);	/* Enable TWI0 interrupt */
}
