#ifndef __USB_H__
#define __USB_H__

#define	EP_CTRL	    0
#define CDC_EP_OUT  1
#define CDC_EP_IN   2
#define	CDC_EP_NOTICE 3
#define NUM_EP      4

#define	EP_SIZE_CTRL 64
#define	EP_SIZE_BULK 64

/* Setup direction bit */
#define	DIR_DEV_TO_HOST	0x80
#define	DIR_HOST_TO_DEV	0x00

/* Standard request */
#define STD_GET_STATUS_ZERO       0x0080
#define STD_GET_STATUS_INTERFACE  0x0081
#define STD_GET_STATUS_ENDPOINT   0x0082

#define STD_CLEAR_FEATURE_ZERO      0x0100
#define STD_CLEAR_FEATURE_INTERFACE 0x0101
#define STD_CLEAR_FEATURE_ENDPOINT  0x0102

#define STD_SET_FEATURE_ZERO        0x0300
#define STD_SET_FEATURE_INTERFACE   0x0301
#define STD_SET_FEATURE_ENDPOINT    0x0302

#define STD_SET_ADDRESS     0x0500
#define STD_GET_DESCRIPTOR  0x0680
#define STD_SET_DESCRIPTOR  0x0700
#define STD_GET_CONFIGURATION 0x0880
#define STD_SET_CONFIGURATION 0x0900
#define STD_GET_INTERFACE     0x0a81
#define STD_SET_INTERFACE     0x0b01
#define STD_SYNCH_FRAME       0x0c82

/* CDC Class request */
#define	CDC_SET_LINECODING	0x2021
#define	CDC_GET_LINECODING	0x21a1
#define	CDC_SET_CTRL_LINE_STATE 0x2221

/* CDC Class notification */
#define	CDC_SERIAL_STATE	0x20a1
#define	bRxCarrier	0x01
#define	bTxCarrier	0x02

/* Descriptor Types */
#define	DESC_DEVICE	1
#define	DESC_CONFIG	2
#define	DESC_STRING	3
#define	DESC_INTERFACE	4
#define	DESC_ENDPOINT	5
#define	DESC_DEV_QUAL	6

struct usb_pkt {
  uint8_t  bmRequestType, bRequest;
  uint16_t wValue, wIndex, wLength;
} __attribute__((packed));

struct cdc_line_coding {
  uint32_t dwDTERate;
  uint8_t  bCharFormat;
  uint8_t  bParitType;
  uint8_t  bDataBits;
} __attribute__((packed));

void udp_init(void);
void udp_disable(void);
int cdc_read(uint8_t *buff, int len);
int cdc_write(uint8_t *buff, int len);
void usb_task(void);

#endif  /* __USB_H__ */
